@@ -1,7 +1,7 @@
 use sqlx_core::bytes::Bytes;
 
 use crate::error::Error;
-use crate::io::Decode;
+use crate::message::{BackendMessage, BackendMessageFormat};
 
 #[derive(Debug)]
 #[repr(u8)]
@@ -21,8 +21,10 @@ pub struct ReadyForQuery {
     pub transaction_status: TransactionStatus,
 }
 
-impl Decode<'_> for ReadyForQuery {
-    fn decode_with(buf: Bytes, _: ()) -> Result<Self, Error> {
+impl BackendMessage for ReadyForQuery {
+    const FORMAT: BackendMessageFormat = BackendMessageFormat::ReadyForQuery;
+
+    fn decode_body(buf: Bytes) -> Result<Self, Error> {
         let status = match buf[0] {
             b'I' => TransactionStatus::Idle,
             b'T' => TransactionStatus::Transaction,
@@ -46,7 +48,7 @@ impl Decode<'_> for ReadyForQuery {
 fn test_decode_ready_for_query() -> Result<(), Error> {
     const DATA: &[u8] = b"E";
 
-    let m = ReadyForQuery::decode(Bytes::from_static(DATA))?;
+    let m = ReadyForQuery::decode_body(Bytes::from_static(DATA))?;
 
     assert!(matches!(m.transaction_status, TransactionStatus::Error));
 
