@@ -16,9 +16,9 @@ impl TransactionManager for PgTransactionManager {
     fn begin(conn: &mut PgConnection) -> BoxFuture<'_, Result<(), Error>> {
         Box::pin(async move {
             let rollback = Rollback::new(conn);
-            let query = begin_ansi_transaction_sql(rollback.conn.transaction_depth);
+            let query = begin_ansi_transaction_sql(rollback.conn.inner.transaction_depth);
             rollback.conn.queue_simple_query(&query)?;
-            rollback.conn.transaction_depth += 1;
+            rollback.conn.inner.transaction_depth += 1;
             rollback.conn.wait_until_ready().await?;
             rollback.defuse();
 
@@ -28,11 +28,11 @@ impl TransactionManager for PgTransactionManager {
 
     fn commit(conn: &mut PgConnection) -> BoxFuture<'_, Result<(), Error>> {
         Box::pin(async move {
-            if conn.transaction_depth > 0 {
-                conn.execute(&*commit_ansi_transaction_sql(conn.transaction_depth))
+            if conn.inner.transaction_depth > 0 {
+                conn.execute(&*commit_ansi_transaction_sql(conn.inner.transaction_depth))
                     .await?;
 
-                conn.transaction_depth -= 1;
+                conn.inner.transaction_depth -= 1;
             }
 
             Ok(())
@@ -41,11 +41,13 @@ impl TransactionManager for PgTransactionManager {
 
     fn rollback(conn: &mut PgConnection) -> BoxFuture<'_, Result<(), Error>> {
         Box::pin(async move {
-            if conn.transaction_depth > 0 {
-                conn.execute(&*rollback_ansi_transaction_sql(conn.transaction_depth))
-                    .await?;
+            if conn.inner.transaction_depth > 0 {
+                conn.execute(&*rollback_ansi_transaction_sql(
+                    conn.inner.transaction_depth,
+                ))
+                .await?;
 
-                conn.transaction_depth -= 1;
+                conn.inner.transaction_depth -= 1;
             }
 
             Ok(())
@@ -53,11 +55,11 @@ impl TransactionManager for PgTransactionManager {
     }
 
     fn start_rollback(conn: &mut PgConnection) {
-        if conn.transaction_depth > 0 {
-            conn.queue_simple_query(&rollback_ansi_transaction_sql(conn.transaction_depth))
+        if conn.inner.transaction_depth > 0 {
+            conn.queue_simple_query(&rollback_ansi_transaction_sql(conn.inner.transaction_depth))
                 .expect("BUG: Rollback query somehow too large for protocol");
 
-            conn.transaction_depth -= 1;
+            conn.inner.transaction_depth -= 1;
         }
     }
 }
