@@ -11,6 +11,34 @@ use crate::types::Oid;
 pub(crate) use sqlx_core::type_info::TypeInfo;
 
 /// Type information for a PostgreSQL type.
+///
+/// ### Note: Implementation of `==` ([`PartialEq::eq()`])
+/// Because `==` on [`TypeInfo`]s has been used throughout the SQLx API as a synonym for type compatibility,
+/// e.g. in the default impl of [`Type::compatible()`][sqlx_core::types::Type::compatible],
+/// some concessions have been made in the implementation.
+///
+/// When comparing two `PgTypeInfo`s using the `==` operator ([`PartialEq::eq()`]),
+/// if one was constructed with [`Self::with_oid()`] and the other with [`Self::with_name()`] or
+/// [`Self::array_of()`], `==` will return `true`:
+///
+/// ```
+/// # use sqlx::postgres::{types::Oid, PgTypeInfo};
+/// // Potentially surprising result, this assert will pass:
+/// assert_eq!(PgTypeInfo::with_oid(Oid(1)), PgTypeInfo::with_name("definitely_not_real"));
+/// ```
+///
+/// Since it is not possible in this case to prove the types are _not_ compatible (because
+/// both `PgTypeInfo`s need to be resolved by an active connection to know for sure)
+/// and type compatibility is mainly done as a sanity check anyway,
+/// it was deemed acceptable to fudge equality in this very specific case.
+///
+/// This also applies when querying with the text protocol (not using prepared statements,
+/// e.g. [`sqlx::raw_sql()`][sqlx_core::raw_sql::raw_sql]), as the connection will be unable
+/// to look up the type info like it normally does when preparing a statement: it won't know
+/// what the OIDs of the output columns will be until it's in the middle of reading the result,
+/// and by that time it's too late.
+///
+/// To compare types for exact equality, use [`Self::type_eq()`] instead.
 #[derive(Debug, Clone, PartialEq)]
 #[cfg_attr(feature = "offline", derive(serde::Serialize, serde::Deserialize))]
 pub struct PgTypeInfo(pub(crate) PgType);
@@ -132,6 +160,8 @@ pub enum PgType {
     // NOTE: Do we want to bring back type declaration by ID? It's notoriously fragile but
     //       someone may have a user for it
     DeclareWithOid(Oid),
+
+    DeclareArrayOf(Arc<PgArrayOf>),
 }
 
 #[derive(Debug, Clone)]
@@ -155,6 +185,13 @@ pub enum PgTypeKind {
     Range(PgTypeInfo),
 }
 
+#[derive(Debug)]
+#[cfg_attr(feature = "offline", derive(serde::Serialize, serde::Deserialize))]
+pub struct PgArrayOf {
+    pub(crate) elem_name: UStr,
+    pub(crate) name: Box<str>,
+}
+
 impl PgTypeInfo {
     /// Returns the corresponding `PgTypeInfo` if the OID is a built-in type and recognized by SQLx.
     pub(crate) fn try_from_oid(oid: Oid) -> Option<Self> {
@@ -233,18 +270,79 @@ impl PgTypeInfo {
     ///
     /// The OID for the type will be fetched from Postgres on use of
     /// a value of this type. The fetched OID will be cached per-connection.
+    ///
+    /// ### Note: Type Names Prefixed with `_`
+    /// In `pg_catalog.pg_type`, Postgres prefixes a type name with `_` to denote an array of that
+    /// type, e.g. `int4[]` actually exists in `pg_type` as `_int4`.
+    ///
+    /// Previously, it was necessary in manual [`PgHasArrayType`][crate::PgHasArrayType] impls
+    /// to return [`PgTypeInfo::with_name()`] with the type name prefixed with `_` to denote
+    /// an array type, but this would not work with schema-qualified names.
+    ///
+    /// As of 0.8, [`PgTypeInfo::array_of()`] is used to declare an array type,
+    /// and the Postgres driver is now able to properly resolve arrays of custom types,
+    /// even in other schemas, which was not previously supported.
+    ///
+    /// It is highly recommended to migrate existing usages to [`PgTypeInfo::array_of()`] where
+    /// applicable.
+    ///
+    /// However, to maintain compatibility, the driver now infers any type name prefixed with `_`
+    /// to be an array of that type. This may introduce some breakages for types which use
+    /// a `_` prefix but which are not arrays.
+    ///
+    /// As a workaround, type names with `_` as a prefix but which are not arrays should be wrapped
+    /// in quotes, e.g.:
+    /// ```
+    /// use sqlx::postgres::PgTypeInfo;
+    /// use sqlx::{Type, TypeInfo};
+    ///
+    /// /// `CREATE TYPE "_foo" AS ENUM ('Bar', 'Baz');`
+    /// #[derive(sqlx::Type)]
+    /// // Will prevent SQLx from inferring `_foo` as an array type.
+    /// #[sqlx(type_name = r#""_foo""#)]
+    /// enum Foo {
+    ///     Bar,
+    ///     Baz
+    /// }
+    ///
+    /// assert_eq!(Foo::type_info().name(), r#""_foo""#);
+    /// ```
     pub const fn with_name(name: &'static str) -> Self {
         Self(PgType::DeclareWithName(UStr::Static(name)))
     }
 
+    /// Create a `PgTypeInfo` of an array from the name of its element type.
+    ///
+    /// The array type OID will be fetched from Postgres on use of a value of this type.
+    /// The fetched OID will be cached per-connection.
+    pub fn array_of(elem_name: &'static str) -> Self {
+        // to satisfy `name()` and `display_name()`, we need to construct strings to return
+        Self(PgType::DeclareArrayOf(Arc::new(PgArrayOf {
+            elem_name: elem_name.into(),
+            name: format!("{elem_name}[]").into(),
+        })))
+    }
+
     /// Create a `PgTypeInfo` from an OID.
     ///
     /// Note that the OID for a type is very dependent on the environment. If you only ever use
     /// one database or if this is an unhandled built-in type, you should be fine. Otherwise,
-    /// you will be better served using [`with_name`](Self::with_name).
+    /// you will be better served using [`Self::with_name()`].
+    ///
+    /// ### Note: Interaction with `==`
+    /// This constructor may give surprising results with `==`.
+    ///
+    /// See [the type-level docs][Self] for details.
     pub const fn with_oid(oid: Oid) -> Self {
         Self(PgType::DeclareWithOid(oid))
     }
+
+    /// Returns `true` if `self` can be compared exactly to `other`.
+    ///
+    /// Unlike `==`, this will return false if
+    pub fn type_eq(&self, other: &Self) -> bool {
+        self.eq_impl(other, false)
+    }
 }
 
 // DEVELOPER PRO TIP: find builtin type OIDs easily by grepping this file
@@ -464,6 +562,9 @@ impl PgType {
             PgType::DeclareWithName(_) => {
                 return None;
             }
+            PgType::DeclareArrayOf(_) => {
+                return None;
+            }
         })
     }
 
@@ -561,9 +662,10 @@ impl PgType {
             PgType::Money => "MONEY",
             PgType::MoneyArray => "MONEY[]",
             PgType::Void => "VOID",
-            PgType::Custom(ty) => &*ty.name,
+            PgType::Custom(ty) => &ty.name,
             PgType::DeclareWithOid(_) => "?",
             PgType::DeclareWithName(name) => name,
+            PgType::DeclareArrayOf(array) => &array.name,
         }
     }
 
@@ -661,9 +763,10 @@ impl PgType {
             PgType::Money => "money",
             PgType::MoneyArray => "_money",
             PgType::Void => "void",
-            PgType::Custom(ty) => &*ty.name,
+            PgType::Custom(ty) => &ty.name,
             PgType::DeclareWithOid(_) => "?",
             PgType::DeclareWithName(name) => name,
+            PgType::DeclareArrayOf(array) => &array.name,
         }
     }
 
@@ -771,13 +874,16 @@ impl PgType {
             PgType::DeclareWithName(name) => {
                 unreachable!("(bug) use of unresolved type declaration [name={name}]");
             }
+            PgType::DeclareArrayOf(array) => {
+                unreachable!(
+                    "(bug) use of unresolved type declaration [array of={}]",
+                    array.elem_name
+                );
+            }
         }
     }
 
     /// If `self` is an array type, return the type info for its element.
-    ///
-    /// This method should only be called on resolved types: calling it on
-    /// a type that is merely declared (DeclareWithOid/Name) is a bug.
     pub(crate) fn try_array_element(&self) -> Option<Cow<'_, PgTypeInfo>> {
         // We explicitly match on all the `None` cases to ensure an exhaustive match.
         match self {
@@ -885,14 +991,50 @@ impl PgType {
                 PgTypeKind::Enum(_) => None,
                 PgTypeKind::Range(_) => None,
             },
-            PgType::DeclareWithOid(oid) => {
-                unreachable!("(bug) use of unresolved type declaration [oid={}]", oid.0);
-            }
+            PgType::DeclareWithOid(_) => None,
             PgType::DeclareWithName(name) => {
-                unreachable!("(bug) use of unresolved type declaration [name={name}]");
+                // LEGACY: infer the array element name from a `_` prefix
+                UStr::strip_prefix(name, "_")
+                    .map(|elem| Cow::Owned(PgTypeInfo(PgType::DeclareWithName(elem))))
             }
+            PgType::DeclareArrayOf(array) => Some(Cow::Owned(PgTypeInfo(PgType::DeclareWithName(
+                array.elem_name.clone(),
+            )))),
         }
     }
+
+    /// Returns `true` if this type cannot be matched by name.
+    fn is_declare_with_oid(&self) -> bool {
+        matches!(self, Self::DeclareWithOid(_))
+    }
+
+    /// Compare two `PgType`s, first by OID, then by array element, then by name.
+    ///
+    /// If `soft_eq` is true and `self` or `other` is `DeclareWithOid` but not both, return `true`
+    /// before checking names.
+    fn eq_impl(&self, other: &Self, soft_eq: bool) -> bool {
+        if let (Some(a), Some(b)) = (self.try_oid(), other.try_oid()) {
+            // If there are OIDs available, use OIDs to perform a direct match
+            return a == b;
+        }
+
+        if soft_eq && (self.is_declare_with_oid() || other.is_declare_with_oid()) {
+            // If we get to this point, one instance is `DeclareWithOid()` and the other is
+            // `DeclareArrayOf()` or `DeclareWithName()`, which means we can't compare the two.
+            //
+            // Since this is only likely to occur when using the text protocol where we can't
+            // resolve type names before executing a query, we can just opt out of typechecking.
+            return true;
+        }
+
+        if let (Some(elem_a), Some(elem_b)) = (self.try_array_element(), other.try_array_element())
+        {
+            return elem_a == elem_b;
+        }
+
+        // Otherwise, perform a match on the name
+        name_eq(self.name(), other.name())
+    }
 }
 
 impl TypeInfo for PgTypeInfo {
@@ -907,6 +1049,13 @@ impl TypeInfo for PgTypeInfo {
     fn is_void(&self) -> bool {
         matches!(self.0, PgType::Void)
     }
+
+    fn type_compatible(&self, other: &Self) -> bool
+    where
+        Self: Sized,
+    {
+        self == other
+    }
 }
 
 impl PartialEq<PgCustomType> for PgCustomType {
@@ -1140,21 +1289,102 @@ impl Display for PgTypeInfo {
 
 impl PartialEq<PgType> for PgType {
     fn eq(&self, other: &PgType) -> bool {
-        if let (Some(a), Some(b)) = (self.try_oid(), other.try_oid()) {
-            // If there are OIDs available, use OIDs to perform a direct match
-            a == b
-        } else if matches!(
-            (self, other),
-            (PgType::DeclareWithName(_), PgType::DeclareWithOid(_))
-                | (PgType::DeclareWithOid(_), PgType::DeclareWithName(_))
-        ) {
-            // One is a declare-with-name and the other is a declare-with-id
-            // This only occurs in the TEXT protocol with custom types
-            // Just opt-out of type checking here
-            true
+        self.eq_impl(other, true)
+    }
+}
+
+/// Check type names for equality, respecting Postgres' case sensitivity rules for identifiers.
+///
+/// https://www.postgresql.org/docs/current/sql-syntax-lexical.html#SQL-SYNTAX-IDENTIFIERS
+fn name_eq(name1: &str, name2: &str) -> bool {
+    // Cop-out of processing Unicode escapes by just using string equality.
+    if name1.starts_with("U&") {
+        // If `name2` doesn't start with `U&` this will automatically be `false`.
+        return name1 == name2;
+    }
+
+    let mut chars1 = identifier_chars(name1);
+    let mut chars2 = identifier_chars(name2);
+
+    while let (Some(a), Some(b)) = (chars1.next(), chars2.next()) {
+        if !a.eq(&b) {
+            return false;
+        }
+    }
+
+    chars1.next().is_none() && chars2.next().is_none()
+}
+
+struct IdentifierChar {
+    ch: char,
+    case_sensitive: bool,
+}
+
+impl IdentifierChar {
+    fn eq(&self, other: &Self) -> bool {
+        if self.case_sensitive || other.case_sensitive {
+            self.ch == other.ch
         } else {
-            // Otherwise, perform a match on the name
-            self.name().eq_ignore_ascii_case(other.name())
+            self.ch.eq_ignore_ascii_case(&other.ch)
         }
     }
 }
+
+/// Return an iterator over all significant characters of an identifier.
+///
+/// Ignores non-escaped quotation marks.
+fn identifier_chars(ident: &str) -> impl Iterator<Item = IdentifierChar> + '_ {
+    let mut case_sensitive = false;
+    let mut last_char_quote = false;
+
+    ident.chars().filter_map(move |ch| {
+        if ch == '"' {
+            if last_char_quote {
+                last_char_quote = false;
+            } else {
+                last_char_quote = true;
+                return None;
+            }
+        } else if last_char_quote {
+            last_char_quote = false;
+            case_sensitive = !case_sensitive;
+        }
+
+        Some(IdentifierChar { ch, case_sensitive })
+    })
+}
+
+#[test]
+fn test_name_eq() {
+    let test_values = [
+        ("foo", "foo", true),
+        ("foo", "Foo", true),
+        ("foo", "FOO", true),
+        ("foo", r#""foo""#, true),
+        ("foo", r#""Foo""#, false),
+        ("foo", "foo.foo", false),
+        ("foo.foo", "foo.foo", true),
+        ("foo.foo", "foo.Foo", true),
+        ("foo.foo", "foo.FOO", true),
+        ("foo.foo", "Foo.foo", true),
+        ("foo.foo", "Foo.Foo", true),
+        ("foo.foo", "FOO.FOO", true),
+        ("foo.foo", "foo", false),
+        ("foo.foo", r#"foo."foo""#, true),
+        ("foo.foo", r#"foo."Foo""#, false),
+        ("foo.foo", r#"foo."FOO""#, false),
+    ];
+
+    for (left, right, eq) in test_values {
+        assert_eq!(
+            name_eq(left, right),
+            eq,
+            "failed check for name_eq({left:?}, {right:?})"
+        );
+        assert_eq!(
+            name_eq(right, left),
+            eq,
+            "failed check for name_eq({right:?}, {left:?})"
+        );
+    }
+}
