@@ -21,10 +21,12 @@ impl PgHasArrayType for Date {
 }
 
 impl Encode<'_, Postgres> for Date {
-    fn encode_by_ref(&self, buf: &mut PgArgumentBuffer) -> IsNull {
-        // DATE is encoded as the days since epoch
-        let days = (*self - PG_EPOCH).whole_days() as i32;
-        Encode::<Postgres>::encode(&days, buf)
+    fn encode_by_ref(&self, buf: &mut PgArgumentBuffer) -> Result<IsNull, BoxDynError> {
+        // DATE is encoded as number of days since epoch (2000-01-01)
+        let days: i32 = (*self - PG_EPOCH).whole_days().try_into().map_err(|_| {
+            format!("value {self:?} would overflow binary encoding for Postgres DATE")
+        })?;
+        Encode::<Postgres>::encode(days, buf)
     }
 
     fn size_hint(&self) -> usize {
